//
//  AddUpdateViewController.swift
//  TaskList
//
//  Created by Merlyn on 2/17/15.
//  Copyright (c) 2015 The App Lady. All rights reserved.
//

import UIKit
import CoreData

class AddUpdateViewController: UIViewController {
    @IBOutlet weak var taskTitle: UITextField!
    @IBOutlet weak var taskDescription: UITextField!
    let context = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    var selectedTask: Task!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        if selectedTask != nil {
            // Set the view's navigation bar's title property
            self.navigationItem.title = "Edit Task"
            
            // Populated the view's text fields with the selectedTask's properties
            taskTitle.text = selectedTask.taskTitle
            taskDescription.text = selectedTask.taskDescription
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    @IBAction func doneButtonTapped(sender: AnyObject) {
        // These statements dismiss the on-screen keyboard
        taskTitle.resignFirstResponder()
        taskDescription.resignFirstResponder()
        
        // Check to see if the Task Title field is empty
        if taskTitle.text.isEmpty {
            
            // The Task Title text field is empty
            self.showAlertViewMessage("The Task Title is required")
            
            // Don't execute remaining code in the function
            return
        }
        
        if selectedTask == nil {
            // Add the new task in the database file
            insertManagedObject()
        } else {
            // Update the existing task in the database file
            updateManagedObject()
        }
    }
    
    func insertManagedObject() {
        let entityDescription = NSEntityDescription.entityForName("Task", inManagedObjectContext: context)
        let task = Task(entity: entityDescription!, insertIntoManagedObjectContext: context)
        var error: NSError?
        
        task.taskTitle = taskTitle.text;
        task.taskDescription = taskDescription.text
        
        if context.save(&error) == true {
            taskTitle.text = nil
            taskDescription.text = nil
        } else {
            println(error?.localizedDescription)
        }

        self.showAlertViewMessage("Task saved")
    }
    
    func updateManagedObject() {
        selectedTask.taskTitle = taskTitle.text
        selectedTask.taskDescription = taskDescription.text
        
        var error: NSError?

        if context.save(&error) == true {
            self.navigationController!.popViewControllerAnimated(true)
        } else {
            println(error?.localizedDescription)
        }
    }
    
    func showAlertViewMessage(msg: String) {
        // Setup the alert view controller
        let alertController = UIAlertController(title: "TaskList", message: msg, preferredStyle: .Alert)
        
        // Setup an action and add it to the alert view controller
        let defaultAction = UIAlertAction(title: "Dismiss", style: .Default, handler: nil)
        alertController.addAction(defaultAction)
        
        // Show the alert view on the device's screen
        self.presentViewController(alertController, animated: true, completion: nil)
    }
    
    override func touchesBegan(touches: NSSet, withEvent event: UIEvent) {
        // These statements dismiss the on-screen keyboard
        taskTitle.resignFirstResponder()
        taskDescription.resignFirstResponder()
    }

}
