//
//  ViewController.swift
//  TaskList
//
//  Created by Merlyn on 2/17/15.
//  Copyright (c) 2015 The App Lady. All rights reserved.
//

import UIKit
import CoreData

class TaskListViewController: UIViewController {
    @IBOutlet weak var searchBar: UISearchBar!
    @IBOutlet weak var tableView: UITableView!
    let context = (UIApplication.sharedApplication().delegate as AppDelegate).managedObjectContext!
    var dataSource = [Task]()

    override func viewDidLoad() {
        super.viewDidLoad()
        fetchManagedObjects()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    override func viewWillAppear(animated: Bool) {
        dataSource.removeAll()
        fetchManagedObjects()
        tableView.reloadData()
    }
    
    func fetchManagedObjects() {
        let freq = NSFetchRequest(entityName: "Task")
        var error: NSError?
        let sortDescriptor = NSSortDescriptor(key: "taskTitle", ascending: true)
        freq.sortDescriptors = [sortDescriptor]
        let fetchedResults = context.executeFetchRequest(freq, error: &error)
        
        if let results = fetchedResults {
            for object in results {
                let row = object as Task
                dataSource.append(row)
            }
        } else {
            println("Could not fetch data from database\n\(error), \(error!.userInfo)")
        }
    }
    
    // Mark: - Table View data source
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // Return the number of rows in the section
        return dataSource.count
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("Cell", forIndexPath: indexPath) as UITableViewCell
        
        // Configure and return the cell
        var task: Task = dataSource[indexPath.row]
        cell.textLabel!.text = task.taskTitle
        cell.detailTextLabel!.text = task.taskDescription
        return cell
    }
    
    func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        if editingStyle == .Delete {
            // Put the swiped table view cell's managed object in a constant
            let taskToDelete = dataSource[indexPath.row]
        
            // Delete the managed object from the database & the table view
            deleteManagedObject(taskToDelete, index: indexPath)
            
            // Tell the table view to animate out the deleted cell
            tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
        }
    }
        
    func deleteManagedObject(let taskToDelete: Task, let index: NSIndexPath) {
        // Tell the context which managed object it will delete from the database
        context.deleteObject(taskToDelete)
        
        var savingError: NSError?
        
        // Delete the the managed object (taskToDelete) from the database
        if !context.save(&savingError) {
            println("Delete managed object operation failed\n \(savingError?.localizedDescription)")
        }
        
        // Delete the managed object from the table view's dataSource object
        dataSource.removeAtIndex(index.row)
    }
    
    // MARK: - Navigation
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject!) {
        if segue.identifier == "idEditTask" {
            if let indexPath = self.tableView.indexPathForSelectedRow() {
                let controller = (segue.destinationViewController as AddUpdateViewController)
                var selectedTask = dataSource[indexPath.row] as Task
                
                // Pass the selected task to the destination view controller
                controller.selectedTask = selectedTask
            }
        }
    }
    
    func unwindSegue() {
    }

    // MARK: - Search Bar delegate
    
    func searchBar(searchBar: UISearchBar, textDidChange searchText: String) {
        searchBar.setShowsCancelButton(true, animated: true)
        
        if !searchText.isEmpty {
            dataSource.removeAll()
            let fetchRequest = NSFetchRequest(entityName: "Task")
            
            fetchRequest.fetchLimit = 25
            fetchRequest.predicate = NSPredicate(format: "taskTitle contains[cd] %@", searchText)
            let sortDescriptor = NSSortDescriptor(key: "taskTitle", ascending: true)
            fetchRequest.sortDescriptors = [sortDescriptor]
            
            var error: NSError? = nil
            if let fetchedResults = context.executeFetchRequest(fetchRequest, error: &error) {
                dataSource = fetchedResults as [Task]
            } else {
                println("Search Request Failed\n \(error?.localizedDescription)")
            }
            
            tableView.reloadData()
        }
    }

    func searchBarCancelButtonClicked(searchBar: UISearchBar!) {
        dataSource.removeAll()
        searchBar.text = nil
        searchBar.showsCancelButton = false
        searchBar.resignFirstResponder()
        fetchManagedObjects()
        tableView.reloadData()
    }

}
